--[[
	Module.....: Minimap
	Description: Replace the default unitframes 
]]
-- ####################################################################################################################
-- ##### Setup and Locals #############################################################################################
-- ####################################################################################################################

---@type string, LUIAddon
local addonName, LUI = ...
local L = LUI.L

---@class UnitFrameModule : LUIModule
local module = LUI:NewModule("Unitframes", "LUIDevAPI", "AceHook-3.0", "AceSerializer-3.0")
module.enableButton = true

module.unitsSpawn = {"player", "target", "focus", "focustarget", "targettarget", "targettargettarget", "pet", "pettarget", "boss", "party", "maintank", "arena", "raid"}

module.units = {"player", "target", "targettarget", "targettargettarget", "focus", "focustarget", "pet", "pettarget", "party", "partytarget", "partypet", "boss", "bosstarget", "maintank", "maintanktarget", "maintanktargettarget", "arena", "arenatarget", "arenapet", "raid"}

module.framelist = {
	player = {"oUF_LUI_player"},
	target = {"oUF_LUI_target"},
	targettarget = {"oUF_LUI_targettarget"},
	targettargettarget = {"oUF_LUI_targettargettarget"},
	focus = {"oUF_LUI_focus"},
	focustarget = {"oUF_LUI_focustarget"},
	pet = {"oUF_LUI_pet"},
	pettarget = {"oUF_LUI_pettarget"},
	party = {},
	partytarget = {},
	partypet = {},
	boss = {},
	bosstarget ={},
	maintank = {},
	maintanktarget = {},
	maintanktargettarget = {},
	arena = {},
	arenatarget = {},
	arenapet = {},
	raid = {},
}

local NameFormats = {
	party = "oUF_LUI_partyUnitButton%d",
	partytarget = "oUF_LUI_partyUnitButton%dtarget",
	partypet = "oUF_LUI_partyUnitButtonpet%d",
	boss = "oUF_LUI_boss%d",
	bosstarget = "oUF_LUI_boss%dtarget",
	maintank = "oUF_LUI_maintankUnitButton%d",
	maintanktarget = "oUF_LUI_maintankUnitButton%dtarget",
	maintanktargettarget = "oUF_LUI_maintankUnitButton%dtargettarget",
	arena = "oUF_LUI_arena%d",
	arenatarget = "oUF_LUI_arena%dtarget",
	arenapet = "oUF_LUI_arenapet%d",
}

local Count = {
	party = 5,
	partytarget = 5,
	partypet = 5,
	boss = 4,
	bosstarget = 4,
	maintank = 3,
	maintanktarget = 3,
	maintanktargettarget = 3,
	arena = 5,
	arenatarget = 5,
	arenapet = 5,
}

-- adding group frames
for k, v in pairs(Count) do
	for i = 1, v do
		module.framelist[k][i] = string.format(NameFormats[k], i)
	end
end

for i = 1, 5 do
	for j = 1, 5 do
		table.insert(module.framelist.raid, "oUF_LUI_raid_25_"..i.."UnitButton"..j)
	end
end

for i = 1, 8 do
	for j = 1, 5 do
		table.insert(module.framelist.raid, "oUF_LUI_raid_40_"..i.."UnitButton"..j)
	end
end

-- ####################################################################################################################
-- ##### Default Settings #############################################################################################
-- ####################################################################################################################

module.childGroups = "tree"
module.defaults = {
	profile = {
		Enable = true,
		Settings = {
			ShowV2Textures = true,
			ShowV2PartyTextures = true,
			ShowV2ArenaTextures = true,
			ShowV2BossTextures = true,
			Castbars = true,
			HideBlizzRaid = false,
			AuratimerFont = "Prototype",
			AuratimerSize = 12,
			AuratimerFlag = "OUTLINE",
		},
		-- ### Player ###
		player = {
			Enable = true,
			Height = 43,
			Width = 250,
			X = -200,
			Y = -200,
			Point = "CENTER",
			Scale = 1,
			HealthBar = {
				Height = 30,
				Width = 250,
			},
			PowerBar = {
				Enable = true,
				Height = 10,
				Width = 250,
				X = 0,
				Y = -32,
			},
			HealthPredictionBar = {
				Enable = false,
				Texture = "LUI_Gradient",
				MyColor = { r = 0, g = 0.5, b = 0, a = 0.25 },
				OtherColor = { r = 0, g = 1, b = 0, a = 0.25 },
			},
			TotalAbsorbBar = {
				Enable = false,
				Texture = "LUI_Gradient",
				MyColor = { r = 0, g = 1, b = 0, a = 0.5 },
			},
			AlternativePowerBar = {
				Enable = false,
				OverPower = false,
				Height = 10,
				Width = 250,
				X = 0,
				Y = -44,
				Color = "By Type",
				IndividualColor = { r = 1, g = 1, b = 1 },
				Texture = "LUI_Gradient",
				TextureBG = "LUI_Gradient",
				BGAlpha = 1,
				BGMultiplier = 0.4,
				Smooth = true,
			},
			AdditionalPowerBar = {
				Enable = true,
				OverPower = true,
				Height = 10,
				Width = 250,
				X = 0,
				Y = -44,
				Color = "By Type",
				IndividualColor = { r = 1, g = 1, b = 1 },
				Texture = "LUI_Minimalist",
				TextureBG = "LUI_Minimalist",
				BGAlpha = 1,
				BGMultiplier = 0.4,
				Smooth = true,
			},
			TotemsBar = {
				Enable = true,
				X = 0,
				Y = 0.5,
				Height = 10,
				Width = 250,
				Texture = "LUI_Gradient",
				Padding = 1,
				Multiplier = 0.35,
				IconScale = 1.5,
				Lock = true,
			},
			RunesBar = {
				Enable = true,
				X = 0,
				Y = 0.5,
				Height = 8,
				Width = 249,
				Texture = "LUI_Gradient",
				Padding = 1,
				Lock = true,
			},
			ClassPowerBar = {
				Enable = true,
				X = 0,
				Y = 0.5,
				Height = 8,
				Width = 250,
				Texture = "LUI_Gradient",
				Padding = 1,
				Lock = true,
			},
			Aura = {
				Buffs = {
					X = -0.5,
					Y = -30,
					InitialAnchor = "BOTTOMRIGHT",
					GrowthY = "DOWN",
					GrowthX = "LEFT",
				},
				Debuffs = {
					InitialAnchor = "BOTTOMLEFT",
					GrowthY = "DOWN",
					GrowthX = "RIGHT",
				},
			},
			Castbar = {
				General = {
					Height = 33,
					Width = 360,
					X = 13,
					Y = 155,
					Latency = true,
				},
				Colors = {
					Latency = { r = 0.11, g = 0.11, b = 0.11, a = 0.74 },
				},
			},
			Portrait = {
				Width = 250,
			},
			LeaderIndicator = { Enable = true, Size = 17, X = 0, Y = 10, Point = "TOPLEFT" },
			GroupRoleIndicator = { Enable = true, Size = 22, X = 15, Y = 10, Point = "TOPRIGHT" },
			RaidMarkerIndicator = { Enable = true, Size = 55, X = 0, Y = 10, Point = "CENTER" },
			RestingIndicator = { Enable = true, Size = 27, X = -12, Y = 13, Point = "TOPLEFT" },
			CombatIndicator = { Enable = false, Size = 27, X = -15, Y = -30, Point = "BOTTOMLEFT" },
			NameText = {
				Enable = false,
				Size = 24,
				Point = "BOTTOMLEFT",
				RelativePoint = "BOTTOMRIGHT",
				Format = "Level + Name",
				Length = "Medium",
				ColorNameByClass = true,
				ColorClassByClass = true,
				ColorLevelByDifficulty = true,
				ShowClassification = true,
			},
			HealthText = {
				Enable = true,
				Size = 28,
				X = 0,
				Y = -31,
				Point = "BOTTOMRIGHT",
				Format = "Standard",
			},
			PowerText = {
				Enable = true,
				Size = 21,
				X = 0,
				Y = -52,
				Outline = "",
				Point = "BOTTOMRIGHT",
				Format = "Standard",
			},
			HealthPercentText = {
				Enable = true,
				Size = 16,
				X = 0,
				Y = 6,
			},
			PowerPercentText = {
				Size = 14,
				X = 0,
				Y = -15,
			},
			HealthMissingText = {
				Size = 15,
				X = -3,
				Y = 0,
				Point = "BOTTOMRIGHT",
				RelativePoint = "BOTTOMRIGHT",
			},
			PowerMissingText = {
				Size = 13,
				X = -3,
				Y = -15,
			},
			AdditionalPowerText = {
				Enable = true,
				Outline = "",
				Size = 14,
				X = 0,
				Y = 0,
				Point = "BOTTOM",
				RelativePoint = "BOTTOM",
				Format = "Standard",
				HideIfFullMana = true,
				Color = "Individual",
				IndividualColor = { r = 1, g = 1, b = 1 },
			},
			AlternativePowerText = {
				Enable = false,
				X = 0,
				Y = 0,
				Format = "Standard",
				Font = "Prototype",
				Size = 10,
				Outline = "",
				Color = "Individual",
				IndividualColor = { r = 1, g = 1, b = 1 },
			},
			ClassPowerText = {
				Enable = true,
				X = 220,
				Y = 0,
				Size = 10,
				Outline = "",
			},
			CombatFeedback = {
				Enable = false,
				Font = "vibrocen",
				Outline = "OUTLINE",
				Size = 20,
				Point = "CENTER",
				RelativePoint = "BOTTOM",
				X = 0,
				Y = 0,
				ShowDamage = true,
				ShowHeal = true,
				ShowImmune = true,
				ShowEnergize = true,
				ShowOther = true,
				MaxAlpha = 0.6,
			},
			PvPText = {
				Enable = true,
				Font = "vibroceb",
				Outline = "",
				Size = 12,
				X = 20,
				Y = 5,
				Color = { r = 1.0, g = 0.1, b = 0.1 },
			},
		},
		--- ### Target ###
		target = {
			Enable = true,
			Height = 43,
			Width = 250,
			X = 200,
			Y = -200,
			Point = "CENTER",
			Scale = 1,
			HealthBar = {
				Height = 30,
				Width = 250,
				Tapping = true,
			},
			PowerBar = {
				Enable = true,
				Height = 10,
				Width = 250,
				X = 0,
				Y = -32,
			},
			HealthPredictionBar = {
				Enable = false,
				Texture = "LUI_Gradient",
				MyColor = { r = 0, g = 0.5, b = 0, a = 0.25 },
				OtherColor = { r = 0, g = 1, b = 0, a = 0.25 },
			},
			TotalAbsorbBar = {
				Enable = false,
				Texture = "LUI_Gradient",
				MyColor = { r = 0, g = 1, b = 0, a = 0.5 },
			},
			ComboPointsBar = {
				Enable = true,
				ShowAlways = false,
				X = 0,
				Y = 0.5,
				Height = 5,
				Width = 249,
				Texture = "LUI_Ruben",
				Padding = 1,
				Multiplier = 0.4,
				IndividualBGColor = true,
				BackgroundColor = { r = 0.23, g = 0.23, b = 0.23 },
			},
			Aura = {
				Buffs = {
					Enable = true,
					ColorByType = false,
					PlayerOnly = false,
					IncludePet = false,
					AuraTimer = false,
					DisableCooldown = false,
					CooldownReverse = true,
					X = -0.5,
					Y = 30,
					InitialAnchor = "TOPLEFT",
					GrowthY = "UP",
					GrowthX = "RIGHT",
					Size = 26,
					Spacing = 2,
					Num = 36,
				},
				Debuffs = {
					Enable = true,
					ColorByType = false,
					PlayerOnly = false,
					IncludePet = false,
					FadeOthers = true,
					AuraTimer = false,
					DisableCooldown = false,
					CooldownReverse = true,
					X = -0.5,
					Y = 60,
					InitialAnchor = "TOPRIGHT",
					GrowthY = "UP",
					GrowthX = "LEFT",
					Size = 26,
					Spacing = 2,
					Num = 36,
				},
			},
			Castbar = {
				General = {
					Height = 33,
					Width = 360,
					X = 13,
					Y = 205,
				},
			},
			Portrait = {
				Width = 250,
			},
			LeaderIndicator = { Enable = true, Size = 17, X = 0, Y = 10, Point = "TOPLEFT" },
			GroupRoleIndicator = { Enable = true, Size = 22, X = 15, Y = 10, Point = "TOPRIGHT" },
			RaidMarkerIndicator = { Enable = true, Size = 55, X = 0, Y = 10, Point = "CENTER" },
			NameText = {
				Size = 25,
				X = 5,
				Point = "BOTTOMLEFT",
				RelativePoint = "BOTTOMRIGHT",
				Format = "Level + Name",
				Length = "Medium",
				ColorNameByClass = true,
				ColorClassByClass = true,
				ColorLevelByDifficulty = true,
				ShowClassification = true,
			},
			HealthText = {
				Enable = true,
				Size = 28,
				X = 0,
				Y = -31,
				RelativePoint = "BOTTOMLEFT",
				Format = "Standard",
			},
			PowerText = {
				Enable = true,
				Size = 21,
				X = 0,
				Y = -51,
				IndividualColor = { r = 0, g = 0, b = 0 },
				RelativePoint = "BOTTOMLEFT",
				Format = "Standard",
			},
			HealthPercentText = {
				Enable = true,
				Size = 16,
			},
			CombatFeedback = {
				Enable = false,
				Font = "vibrocen",
				Outline = "OUTLINE",
				Size = 20,
				Point = "CENTER",
				RelativePoint = "BOTTOM",
				X = 0,
				Y = 0,
				ShowDamage = true,
				ShowHeal = true,
				ShowImmune = true,
				ShowEnergize = true,
				ShowOther = true,
				MaxAlpha = 0.6,
			},
		},
		--- ### Player Pet
		pet = {
			Enable = true,
			Height = 43,
			Width = 130,
			X = 0,
			Y = -200,
			Point = "CENTER",
			Scale = 1,
			HealthBar = {
				Height = 30,
				Width = 130,
			},
			PowerBar = {
				Enable = true,
				X = 0,
				Y = -32,
				Color = "By Type",
			},
			HealthPredictionBar = {
				Enable = false,
				Texture = "LUI_Gradient",
				MyColor = { r = 0, g = 0.5, b = 0, a = 0.25 },
				OtherColor = { r = 0, g = 1, b = 0, a = 0.25 },
			},
			TotalAbsorbBar = {
				Enable = false,
				Texture = "LUI_Gradient",
				MyColor = { r = 0, g = 1, b = 0, a = 0.5 },
			},
			Castbar = {
				General = {
					Enable = false,
					Height = 26,
					Width = 130,
				},
			},
			Aura = {
				Buffs = {
					Enable = false,
					ColorByType = false,
					PlayerOnly = false,
					IncludePet = false,
					AuraTimer = false,
					DisableCooldown = false,
					CooldownReverse = true,
					X = -0.5,
					Y = 30,
					InitialAnchor = "TOPLEFT",
					GrowthY = "UP",
					GrowthX = "RIGHT",
					Size = 26,
					Spacing = 2,
					Num = 36,
				},
				Debuffs = {
					Enable = false,
					ColorByType = false,
					PlayerOnly = false,
					IncludePet = false,
					FadeOthers = false,
					AuraTimer = false,
					DisableCooldown = false,
					CooldownReverse = true,
					X = -0.5,
					Y = 60,
					InitialAnchor = "TOPRIGHT",
					GrowthY = "UP",
					GrowthX = "LEFT",
					Size = 26,
					Spacing = 2,
					Num = 36,
				},
			},
			RaidMarkerIndicator = { Enable = false, Size = 55, X = 0, Y = 10, Point = "CENTER" },
			NameText = {
				Enable = true,
				Size = 19,
				X = 0,
				Y = -22,
				Length = "Medium",
			},
			HealthPercentText = {
				Enable = true,
				Size = 16,
				X = 0,
				Y = 6,
			},
			CombatFeedback = {
				Enable = false,
				Font = "vibrocen",
				Outline = "OUTLINE",
				Size = 16,
				Point = "CENTER",
				RelativePoint = "BOTTOM",
				X = 0,
				Y = 0,
				ShowDamage = true,
				ShowHeal = true,
				ShowImmune = true,
				ShowEnergize = true,
				ShowOther = true,
				MaxAlpha = 0.6,
			},
		},
		--- ### Focus
		focus = {
			Enable = true,
			Height = 24,
			Width = 200,
			X = -435,
			Y = -250,
			Point = "CENTER",
			Scale = 1,
			HealthBar = {
				Height = 24,
				Width = 200,
				Color = "By Class",
			},
			PowerBar = {
				Height = 10,
				Width = 200,
			},
			HealthPredictionBar = {
				Enable = false,
				Texture = "LUI_Gradient",
				MyColor = { r = 0, g = 0.5, b = 0, a = 0.25 },
				OtherColor = { r = 0, g = 1, b = 0, a = 0.25 },
			},
			TotalAbsorb = {
				Enable = false,
				Texture = "LUI_Gradient",
				MyColor = { r = 0, g = 1, b = 0, a = 0.5 },
			},
			Aura = {
				Buffs = {
					Enable = false,
					ColorByType = false,
					PlayerOnly = false,
					IncludePet = false,
					AuraTimer = false,
					DisableCooldown = false,
					CooldownReverse = true,
					X = -0.5,
					Y = -30,
					InitialAnchor = "BOTTOMRIGHT",
					GrowthY = "DOWN",
					GrowthX = "LEFT",
					Size = 26,
					Spacing = 2,
				},
				Debuffs = {
					Enable = false,
					ColorByType = false,
					PlayerOnly = false,
					IncludePet = false,
					FadeOthers = true,
					AuraTimer = false,
					DisableCooldown = false,
					CooldownReverse = true,
					X = -0.5,
					Y = -60,
					InitialAnchor = "BOTTOMLEFT",
					GrowthY = "DOWN",
					GrowthX = "RIGHT",
					Size = 26,
					Spacing = 2,
					Num = 36,
				},
			},
			NameText = {
				X = 5,
				Point = "LEFT",
				RelativePoint = "LEFT",
				Length = "Medium",
			},
			HealthPercentText = {
				Enable = true,
				Size = 14,
				X = -5,
				Y = 0,
				Point = "RIGHT",
				RelativePoint = "RIGHT",
				ShowDead = false,
			},
			CombatFeedback = {
				Enable = false,
				Font = "vibrocen",
				Outline = "OUTLINE",
				Size = 16,
				Point = "CENTER",
				RelativePoint = "BOTTOM",
				X = 0,
				Y = 0,
				ShowDamage = true,
				ShowHeal = true,
				ShowImmune = true,
				ShowEnergize = true,
				ShowOther = true,
				MaxAlpha = 0.6,
			},
		},
		--- ### Party
		party = {
			Enable = true,
			UseBlizzard = false,
			Height = 43,
			Width = 170,
			X = 150,
			Y = 100,
			Scale = 1,
			Point = "LEFT",
			GrowDirection = "BOTTOM",
			Padding = 50,
			ShowPlayer = false,
			ShowInRaid = false,
			ShowInRealParty = false,
			RangeFade = true,
			HealthBar = {
				Height = 30,
				Width = 170,
			},
			PowerBar = {
				Enable = true,
				Height = 10,
				Width = 170,
				X = 0,
				Y = -32,
			},
			HealthPredictionBar = {
				Enable = false,
				Texture = "LUI_Gradient",
				MyColor = { r = 0, g = 0.5, b = 0, a = 0.25 },
				OtherColor = { r = 0, g = 1, b = 0, a = 0.25 },
			},
			TotalAbsorb = {
				Enable = false,
				Texture = "LUI_Gradient",
				MyColor = { r = 0, g = 1, b = 0, a = 0.5 },
			},
			Aura = {
				Buffs = {
					Enable = false,
					ColorByType = false,
					PlayerOnly = false,
					IncludePet = false,
					AuraTimer = false,
					DisableCooldown = false,
					CooldownReverse = true,
					X = 0,
					Y = -42,
					InitialAnchor = "BOTTOMLEFT",
					GrowthY = "DOWN",
					GrowthX = "RIGHT",
					Size = 26,
					Spacing = 2,
				},
				Debuffs = {
					Enable = true,
					ColorByType = true,
					PlayerOnly = false,
					IncludePet = false,
					FadeOthers = false,
					AuraTimer = false,
					DisableCooldown = false,
					CooldownReverse = true,
					X = 35,
					Y = -5,
					InitialAnchor = "RIGHT",
					GrowthY = "DOWN",
					GrowthX = "RIGHT",
					Size = 26,
					Spacing = 2,
					Num = 36,
				},
			},
			Castbar = {
				General = {
					Height = 20,
					Width = 100,
					X = 10,
					Y = 0,
					Icon = false,
				},
			},
			LeaderIndicator = { Enable = true, Size = 17, X = 0, Y = 10, Point = "TOPLEFT" },
			GroupRoleIndicator = { Enable = true, Size = 22, X = 15, Y = 10, Point = "TOPRIGHT" },
			RaidMarkerIndicator = { Enable = true, Size = 55, X = 0, Y = 10, Point = "CENTER" },
			ReadyCheckIndicator = { Enable = true, Size = 20, X = 0, Y = 0, Point = "CENTER" },
			NameText = {
				Size = 19,
				X = 0,
				Y = -20,
				Outline = "OUTLINE",
				Length = "Medium",
				ColorNameByClass = true,
			},
			HealthPercentText = {
				Enable = true,
				Size = 16,
				X = 0,
				Y = 6,
			},
		},
		--- ### Party Pets
		partypet = {
			Enable = true,
			Height = 24,
			Width = 130,
			X = -15,
			Y = -10,
			Point = "TOPLEFT",
			RelativePoint = "BOTTOMLEFT",
			HealthBar = {
				Color = "By Class",
			},
			PowerBar = {
				Height = 5,
				Width = 130,
			},
			HealthPredictionBar = {
				Enable = false,
				Texture = "LUI_Gradient",
				MyColor = { r = 0, g = 0.5, b = 0, a = 0.25 },
				OtherColor = { r = 0, g = 1, b = 0, a = 0.25 },
			},
			TotalAbsorb = {
				Enable = false,
				Texture = "LUI_Gradient",
				MyColor = { r = 0, g = 1, b = 0, a = 0.5 },
			},
			Aura = {
				Debuffs = {
					X = -0.5,
					Y = -30,
					InitialAnchor = "RIGHT",
					GrowthY = "UP",
					GrowthX = "LEFT",
					Size = 18,
					Num = 8,
				},
			},
		},
		--- ### Raid
		raid = {
			Enable = true,
			UseBlizzard = false,
			Height = 33,
			Width = 77.5,
			X = -28.5,
			Y = 40.5,
			Point = "BOTTOMRIGHT",
			Padding = 4,
			GroupPadding = 4,
			Border = {
				Aggro = true,
				Target = true,
			},
			HealthBar = {
				Height = 26,
				Width = 77.5,
			},
			PowerBar = {
				Enable = true,
				Height = 5,
				Width = 77.5,
				X = 0,
				Y = -28,
			},
			HealthPredictionBar = {
				Enable = false,
				Texture = "LUI_Gradient",
				MyColor = { r = 0, g = 0.5, b = 0, a = 0.25 },
				OtherColor = { r = 0, g = 1, b = 0, a = 0.25 },
			},
			TotalAbsorb = {
				Enable = false,
				Texture = "LUI_Gradient",
				MyColor = { r = 0, g = 1, b = 0, a = 0.5 },
			},
			CornerAura = {
				Enable = true,
				Size = 8,
				Inset = 1,
			},
			RaidDebuff = {
				Enable = true,
				Size = 16,
			},
			Portrait = {
				Height = 33,
				Width = 77.5,
			},
			LeaderIndicator = { Enable = false, Size = 14, X = 0, Y = 4, Point = "TOPLEFT" },
			GroupRoleIndicator = { Enable = false, Size = 14, X = 2, Y = 2, Point = "TOPRIGHT" },
			RaidMarkerIndicator = { Enable = false, Size = 14, X = 0, Y = 4, Point = "CENTER" },
			ReadyCheckIndicator = { Enable = true, Size = 20, X = 0, Y = 0, Point = "CENTER" },
			NameText = {
				Size = 12,
				Length = "Medium",
				ColorNameByClass = true,
			},
			HealthText = {
				Size = 12,
				X = 0,
				Y = 0,
				Color = "By Class",
				ShowAlways = false,
				Point = "CENTER",
				RelativePoint = "CENTER",
				Format = "Standard Short",
				ShowDead = true,
			},
			PowerText = {
				Size = 12,
				X = 0,
				Y = 3,
				ShowFull = false,
				ShowEmpty = false,
				Point = "CENTER",
				RelativePoint = "BOTTOM",
				Format = "Standard Short",
			},
			HealthPercentText = {
				Size = 12,
				Color = "By Class",
				ShowDead = false,
			},
			PowerPercentText = {
				Size = 12,
				X = 2,
				Y = 3,
				Color = "By Class",
				RelativePoint = "BOTTOM",
			},
			Castbar = {
				General = {
					Enable = false,
					Icon = false,
				},
			},
		},
		--- ### Boss
		boss = {
			Enable = true,
			UseBlizzard = false,
			Height = 24,
			Width = 130,
			X = -25,
			Y = -250,
			Scale = 1,
			Point = "TOPRIGHT",
			GrowDirection = "BOTTOM",
			Padding = 6,
			Aura = {
				Buffs = {
					X = 0,
					Y = -42,
					InitialAnchor = "BOTTOMLEFT",
					GrowthY = "DOWN",
					GrowthX = "RIGHT",
				},
				Debuffs = {
					Enable = true,
					ColorByType = true,
					PlayerOnly = true,
					FadeOthers = true,
					X = 35,
					Y = -5,
					InitialAnchor = "RIGHT",
					GrowthY = "DOWN",
					GrowthX = "RIGHT",
				},
			},
			Castbar = {
				General = {
					Enable = false,
					Height = 20,
					Width = 140,
					X = -140,
					Y = -35,
					Icon = false,
				},
			},
		},
		--- ## Maintank
		maintank = {
			Enable = false,
			Height = 24,
			Width = 130,
			X = -10,
			Y = 350,
			Scale = 1,
			Point = "BOTTOMRIGHT",
			GrowDirection = "BOTTOM",
			Padding = 6,
			HealthPredictionBar = {
				Enable = false,
				Texture = "LUI_Gradient",
				MyColor = { r = 0, g = 0.5, b = 0, a = 0.25 },
				OtherColor = { r = 0, g = 1, b = 0, a = 0.25 },
			},
			TotalAbsorb = {
				Enable = false,
				Texture = "LUI_Gradient",
				MyColor = { r = 0, g = 1, b = 0, a = 0.5 },
			},
			Aura = {
				Buffs = {
					InitialAnchor = "BOTTOMLEFT",
					Size = 18,
				},
				Debuffs = {
					X = 35,
					Y = -5,
					InitialAnchor = "LEFT",
					GrowthY = "UP",
					GrowthX = "RIGHT",
					Size = 18,
					Num = 8,
				},
			},
			Castbar = {
				General = {
					Enable = false,
					Icon = false,
				},
			},
			RaidMarkerIndicator = { Enable = false, Size = 55, X = 0, Y = 0, Point = "CENTER" },
		},
		-- ### Arena
		arena = {
			Enable = true,
			UseBlizzard = false,
			Height = 43,
			Width = 170,
			X = -150,
			Y = 100,
			Scale = 1,
			Point = "RIGHT",
			GrowDirection = "BOTTOM",
			Padding = 50,
			HealthBar = {
				Height = 30,
				Width = 170,
			},
			PowerBar = {
				Enable = true,
				Height = 10,
				Width = 170,
				X = 0,
				Y = -32,
			},
			Aura = {
				Buffs = {
					X = 0,
					Y = -42,
					InitialAnchor = "BOTTOMLEFT",
					GrowthY = "DOWN",
					GrowthX = "RIGHT",
				},
				Debuffs = {
					Enable = true,
					ColorByType = true,
					PlayerOnly = true,
					FadeOthers = true,
					X = 35,
					Y = -5,
					InitialAnchor = "RIGHT",
					GrowthY = "DOWN",
					GrowthX = "RIGHT",
				},
			},
			Castbar = {
				General = {
					Height = 20,
					Width = 100,
					X = -10,
					Y = 0,
					Icon = false,
				},
			},
			NameText = {
				Size = 19,
				X = 0,
				Y = -20,
				Outline = "OUTLINE",
				Length = "Medium",
				ColorNameByClass = true,
			},

			HealthPercentText = {
				Enable = true,
				Size = 16,
				X = 0,
				Y = 6,
			},
		},
		--- ### Arena Pets
		arenapet = {
			Enable = true,
			Height = 24,
			Width = 130,
			X = 15,
			Y = -10,
			Point = "TOPRIGHT",
			RelativePoint = "BOTTOMRIGHT",
			Aura = {
				Buffs = {
					X = 0,
					Y = -42,
					InitialAnchor = "BOTTOMLEFT",
					GrowthY = "DOWN",
					GrowthX = "RIGHT",
				},
				Debuffs = {
					Enable = false,
					ColorByType = true,
					PlayerOnly = true,
					IncludePet = true,
					FadeOthers = true,
					X = 35,
					Y = -5,
					InitialAnchor = "RIGHT",
					GrowthY = "DOWN",
					GrowthX = "RIGHT",
				},
			},
			HealthPercentText = {
				Size = 14,
				X = -5,
				Y = 0,
				Point = "RIGHT",
				RelativePoint = "RIGHT",
			},
		},
		--- ### Target / Target of Target Section ###
		--- ### Pet Target
		pettarget = {
			Enable = true,
			Height = 24,
			Width = 130,
			X = 0,
			Y = -160,
			Point = "CENTER",
			Scale = 1,
		},
		--- ### Party Target
		partytarget = {
			Enable = true,
			Height = 24,
			Width = 130,
			X = 8,
			Y = -8,
			Point = "TOPLEFT",
			RelativePoint = "BOTTOMRIGHT",
			NameText = {
				X = 5,
				Y = 1,
				Point = "LEFT",
				RelativePoint = "LEFT",
			},
			HealthPercentText = {
				Size = 16,
				X = 0,
				Y = 6,
			},
		},
		--- ### FocusTarget
		focustarget = {
			Enable = false,
			Height = 24,
			Width = 200,
			X = -465,
			Y = -285,
			Point = "CENTER",
			Scale = 1,
			HealthBar = {
				Height = 24,
				Width = 200,
				Color = "By Class",
			},
			PowerBar = {
				Height = 10,
				Width = 200,
			},
			HealthPredictionBar = {
				Enable = false,
				Texture = "LUI_Gradient",
				MyColor = { r = 0, g = 0.5, b = 0, a = 0.25 },
				OtherColor = { r = 0, g = 1, b = 0, a = 0.25 },
			},
			TotalAbsorb = {
				Enable = false,
				Texture = "LUI_Gradient",
				MyColor = { r = 0, g = 1, b = 0, a = 0.5 },
			},
			Aura = {
				Buffs = {
					InitialAnchor = "BOTTOMRIGHT",
					GrowthY = "DOWN",
					GrowthX = "LEFT",
				},
				Debuffs = {
					FadeOthers = true,
					X = -0.5,
					Y = -60,
					InitialAnchor = "BOTTOMLEFT",
					GrowthY = "DOWN",
					GrowthX = "RIGHT",
				},
			},
			NameText = {
				X = 5,
				Y = 0,
				Point = "LEFT",
				RelativePoint = "LEFT",
				Length = "Medium",
			},
			HealthPercentText = {
				Enable = true,
				Size = 14,
				X = -5,
				Y = 0,
				Point = "RIGHT",
				RelativePoint = "RIGHT",
				ShowDead = false,
			},
		},
		--- ### Boss Target
		bosstarget = {
			Enable = false,
			Height = 24,
			Width = 130,
			X = -8,
			Y = -8,
			Point = "TOPRIGHT",
			RelativePoint = "BOTTOMLEFT",
			NameText = {
				X = 5,
				Y = 1,
				Point = "LEFT",
				RelativePoint = "LEFT",
			},
			HealthPercentText = {
				Size = 16,
				X = 0,
				Y = 6,
			},
		},
		--- #MainTank Target
		maintanktarget = {
			Enable = true,
			Height = 24,
			Width = 130,
			X = -8,
			Y = 0,
			Point = "TOPRIGHT",
			RelativePoint = "TOPLEFT",
		},
		--- #Arena Target
		arenatarget = {
			Enable = true,
			Height = 24,
			Width = 130,
			X = -8,
			Y = -8,
			Point = "TOPRIGHT",
			RelativePoint = "BOTTOMLEFT",
			NameText = {
				X = 5,
				Y = 1,
				Point = "LEFT",
				RelativePoint = "LEFT",
			},
			HealthPercentText = {
				Size = 16,
				X = 0,
				Y = 6,
			},
		},
		--- ### Target of Target
		targettarget = {
			Enable = true,
			Height = 24,
			Width = 200,
			X = 435,
			Y = -250,
			Point = "CENTER",
			Scale = 1,
			HealthBar = {
				Height = 24,
				Width = 200,
				Color = "By Class",
				Texture = "LUI_Gradient",
				TextureBG = "LUI_Gradient",
			},
			PowerBar = {
				Height = 5,
				Width = 200,
			},
			HealthPredictionBar = {
				Enable = false,
				Texture = "LUI_Gradient",
				MyColor = { r = 0, g = 0.5, b = 0, a = 0.25 },
				OtherColor = { r = 0, g = 1, b = 0, a = 0.25 },
			},
			TotalAbsorb = {
				Enable = false,
				Texture = "LUI_Gradient",
				MyColor = { r = 0, g = 1, b = 0, a = 0.5 },
			},
			Aura = {
				Buffs = {
					Enable = false,
					ColorByType = false,
					PlayerOnly = false,
					IncludePet = false,
					AuraTimer = false,
					DisableCooldown = false,
					CooldownReverse = true,
					X = -0.5,
					Y = 30,
					InitialAnchor = "TOPLEFT",
					GrowthY = "UP",
					GrowthX = "RIGHT",
					Size = 26,
					Spacing = 2,
					Num = 36,
				},
				Debuffs = {
					Enable = false,
					ColorByType = false,
					PlayerOnly = false,
					IncludePet = false,
					FadeOthers = false,
					AuraTimer = false,
					DisableCooldown = false,
					CooldownReverse = true,
					X = -0.5,
					Y = 60,
					InitialAnchor = "TOPRIGHT",
					GrowthY = "UP",
					GrowthX = "LEFT",
					Size = 26,
					Spacing = 2,
					Num = 36,
				},
			},
			NameText = {
				X = 5,
				Point = "LEFT",
				RelativePoint = "LEFT",
				Length = "Medium",
			},
			HealthPercentText = {
				Enable = true,
				Size = 14,
				X = -5,
				Y = 0,
				Point = "RIGHT",
				RelativePoint = "RIGHT",
				ShowDead = false,
			},
			CombatFeedback = {
				Enable = false,
				Font = "vibrocen",
				Outline = "OUTLINE",
				Size = 16,
				Point = "CENTER",
				RelativePoint = "BOTTOM",
				X = 0,
				Y = 0,
				ShowDamage = true,
				ShowHeal = true,
				ShowImmune = true,
				ShowEnergize = true,
				ShowOther = true,
				MaxAlpha = 0.6,
			},
		},
		--- # Target of Target of Target
		targettargettarget = {
			Enable = false,
			Height = 24,
			Width = 200,
			X = 465,
			Y = -285,
			Point = "CENTER",
			Scale = 1,
			HealthBar = {
				Height = 24,
				Width = 200,
				Color = "By Class",
			},
			Aura = {
				Buffs = {
					X = -0.5,
					Y = -30,
					InitialAnchor = "BOTTOMRIGHT",
					GrowthY = "DOWN",
					GrowthX = "LEFT",
				},
				Debuffs = {
					X = -0.5,
					Y = -60,
					InitialAnchor = "BOTTOMLEFT",
					GrowthY = "DOWN",
					GrowthX = "RIGHT",
				},
			},
			NameText = {
				X = 5,
				Point = "LEFT",
				RelativePoint = "LEFT",
				Length = "Medium",
			},
			HealthPercentText = {
				Enable = true,
				Size = 14,
				X = -5,
				IndividualColor = { r = 1, g = 1, b = 1 },
				Point = "RIGHT",
				RelativePoint = "RIGHT",
				ShowDead = false,
			},
		},
		--- ### MainTank Target of Target
		maintanktargettarget = {
			X = -146,
			Y = 0,
		},
		['**'] = {
			Enable = false,
			Height = 24,
			Width = 130,
			X = -100,
			Y = 0,
			Point = "TOPRIGHT",
			RelativePoint = "TOPLEFT",
			Border = {
				Aggro = false,
				EdgeFile = "glow",
				EdgeSize = 5,
				Insets = { Left = 3, Right = 3, Top = 3, Bottom = 3 },
				Color = { r = 0, g = 0, b = 0, a = 1 },
			},
			Backdrop = {
				Texture = "Blizzard Tooltip",
				Padding = { Left = -4, Right = 4, Top = 4, Bottom = -4 },
				Color = { r = 0, g = 0, b = 0, a = 1 },
			},
			HealthBar = {
				Height = 24,
				Width = 130,
				X = 0,
				Y = 0,
				Color = "Individual",
				Texture = "LUI_Gradient",
				TextureBG = "LUI_Gradient",
				BGAlpha = 1,
				BGMultiplier = 0.4,
				BGInvert = false,
				Smooth = true,
				IndividualColor = { r = 0.25, g = 0.25, b = 0.25 },
			},
			PowerBar = {
				Enable = false,
				Height = 10,
				Width = 130,
				X = 0,
				Y = -26,
				Color = "By Class",
				Texture = "LUI_Minimalist",
				TextureBG = "LUI_Minimalist",
				BGAlpha = 1,
				BGMultiplier = 0.4,
				BGInvert = false,
				Smooth = true,
				IndividualColor = { r = 0.8, g = 0.8, b = 0.8 },
			},
			Aura = {
				Buffs = {
					Enable = false,
					ColorByType = false,
					PlayerOnly = false,
					IncludePet = false,
					AuraTimer = false,
					DisableCooldown = false,
					CooldownReverse = true,
					X = -0.5,
					Y = 30,
					InitialAnchor = "TOPLEFT",
					GrowthY = "UP",
					GrowthX = "RIGHT",
					Size = 26,
					Spacing = 2,
					Num = 8,
				},
				Debuffs = {
					Enable = false,
					ColorByType = false,
					PlayerOnly = false,
					IncludePet = false,
					FadeOthers = false,
					AuraTimer = false,
					DisableCooldown = false,
					CooldownReverse = true,
					X = -0.5,
					Y = 60,
					InitialAnchor = "TOPRIGHT",
					GrowthY = "UP",
					GrowthX = "LEFT",
					Size = 26,
					Spacing = 2,
					Num = 36,
				},
			},
			Castbar = {
				General = {
					Enable = true,
					Height = 26,
					Width = 200,
					X = 0,
					Y = 70,
					Point = "BOTTOM",
					Texture = "LUI_Gradient",
					TextureBG = "LUI_Minimalist",
					IndividualColor = false,
					Icon = true,
					Shield = true,
				},
				NameText = {
					Enable = true,
					Font = "Prototype",
					Size = 15,
					Outline = "",
					OffsetX = 5,
					OffsetY = 1,
				},
				TimeText = {
					Enable = true,
					ShowMax = true,
					Font = "Prototype",
					Outline = "",
					Size = 13,
					OffsetX = -5,
					OffsetY = 1,
				},
				Border = {
					Texture = "glow",
					Thickness = 4,
					Inset = { left = 3, right = 3, top = 3, bottom = 3 },
				},
				Colors = {
					Bar = { r = 0.13, g = 0.59, b = 1, a = 0.68 },
					Background = { r = 0.15, g = 0.15, b = 0.15, a = 0.67 },
					Border = { r = 0, g = 0, b = 0, a = 0.7 },
					Shield = { r = 0.5, g = 0, b = 0, a = 0.1 },
					Name = { r = 0.9, g = 0.9, b = 0.9 },
					Time = { r = 0.9, g = 0.9, b = 0.9 },
				},
				Shield = {
					Enable = true,
					Text = true,
					IndividualColor = false,
					BarColor = { r = 0.13, g = 0.59, b = 1, a = 0.68 },
					IndividualBorder = true,
					Color = { r = 0.13, g = 0.59, b = 1, a = 0.68 },
					Border = false,
					Texture = "glow",
					Thickness = 4,
					Inset = { left = 3, right = 3, top = 3, bottom = 3 },
				},
			},
			Portrait = {
				Enable = false,
				Height = 43,
				Width = 90,
				X = 0,
				Y = 0,
				Alpha = 0.5,
			},
			LeaderIndicator = { Enable = false, Size = 17, X = 0, Y = 0, Point = "TOPLEFT" },
			GroupRoleIndicator = { Enable = false, Size = 22, X = 15, Y = 10, Point = "TOPRIGHT" },
			PvPIndicator = { Enable = false, Size = 35, X = -12, Y = 10, Point = "TOPLEFT" },
			RaidMarkerIndicator = { Enable = true, Size = 55, X = 0, Y = 0, Point = "CENTER" },
			NameText = {
				Enable = true,
				Font = "Prototype",
				Outline = "",
				Size = 15,
				X = 0,
				Y = 0,
				IndividualColor = { r = 1, g = 1, b = 1 },
				Point = "CENTER",
				RelativePoint = "CENTER",
				Format = "Name",
				Length = "Short",
				ColorNameByClass = false,
				ColorClassByClass = false,
				ColorLevelByDifficulty = false,
				ShowClassification = false,
				ShortClassification = false,
			},
			HealthText = {
				Enable = false,
				Font = "Prototype",
				Outline = "",
				Size = 24,
				X = 0,
				Y = -43,
				ShowAlways = true,
				ShowDead = false,
				IndividualColor = { r = 1, g = 1, b = 1 },
				Color = "Individual",
				Point = "BOTTOMLEFT",
				RelativePoint = "BOTTOMRIGHT",
				Format = "Absolut Short",
			},
			PowerText = {
				Enable = false,
				Font = "Prototype",
				Outline = "",
				Size = 24,
				X = 0,
				Y = -66,
				ShowFull = true,
				ShowEmpty = true,
				IndividualColor = { r = 1, g = 1, b = 1 },
				Color = "By Class",
				Point = "BOTTOMLEFT",
				RelativePoint = "BOTTOMRIGHT",
				Format = "Absolut Short",
			},
			HealthPercentText = {
				Enable = false,
				Font = "Prototype",
				Outline = "",
				Size = 24,
				X = 0,
				Y = 0,
				ShowAlways = false,
				ShowDead = true,
				IndividualColor = { r = 1, g = 1, b = 1 },
				Color = "Individual",
				Point = "CENTER",
				RelativePoint = "CENTER",
			},
			PowerPercentText = {
				Enable = false,
				Font = "Prototype",
				Outline = "",
				Size = 24,
				X = 0,
				Y = 0,
				ShowFull = false,
				ShowEmpty = false,
				IndividualColor = { r = 1, g = 1, b = 1 },
				Color = "Individual",
				Point = "CENTER",
				RelativePoint = "CENTER",
			},
			HealthMissingText = {
				Enable = false,
				Font = "Prototype",
				Outline = "",
				Size = 24,
				X = 0,
				Y = 0,
				ShortValue = true,
				ShowAlways = false,
				IndividualColor = { r = 1, g = 1, b = 1 },
				Color = "Individual",
				Point = "RIGHT",
				RelativePoint = "RIGHT",
			},
			PowerMissingText = {
				Enable = false,
				Font = "Prototype",
				Outline = "",
				Size = 24,
				X = 0,
				Y = 0,
				ShortValue = true,
				ShowFull = false,
				ShowEmpty = false,
				IndividualColor = { r = 1, g = 1, b = 1 },
				Color = "Individual",
				Point = "RIGHT",
				RelativePoint = "RIGHT",
			},
			Fader = {
				Casting = true,
				Combat = true,
				Enable = false,
				Health = true,
				HealthClip = 1.0,
				Hover = true,
				HoverAlpha = 0.75,
				InAlpha = 1.0,
				OutAlpha = 0.1,
				OutDelay = 0.0,
				OutTime = 1.5,
				Power = true,
				PowerClip = 0.9,
				Targeting = true,
				UseGlobalSettings = true,
			},
		},
		--- ### Colors. Mostly deprecated and will be using the color module.
		Colors = {
			Class = {
				WARRIOR = {1, 0.78, 0.55},
				PRIEST = {0.9, 0.9, 0.9},
				DRUID = {1, 0.44, 0.15},
				HUNTER = {0.22, 0.91, 0.18},
				MAGE = {0.12, 0.58, 0.89},
				PALADIN = {0.96, 0.21, 0.73},
				SHAMAN = {0.04, 0.39, 0.98},
				WARLOCK = {0.57, 0.22, 1},
				ROGUE = {0.95, 0.86, 0.16},
				DEATHKNIGHT = {0.8, 0.1, 0.1},
				MONK = {0.00, 1.00, 0.59},
				DEMONHUNTER = {0.64, 0.19, 0.79},
			},
			--[[ This section shows the actual real Blizzard colors.
				 Some people have asked for them so making it easier
				 for those people to use them.
			Class = {
				WARRIOR = {0.78, 0.61, 0.43},
				PRIEST = {1.00, 1.00, 1.00},
				DRUID = {1.00, 0.49, 0.04},
				HUNTER = {0.67, 0.83, 0.45},
				MAGE = {0.41, 0.80, 0.94},
				PALADIN = {0.96, 0.55, 0.73},
				SHAMAN = {0.00, 0.44, 0.87},
				WARLOCK = {0.58, 0.51, 0.79},
				ROGUE = {1.00, 0.96, 0.41},
				DEATHKNIGHT = {0.77, 0.12, 0.23},
				MONK = {0.00, 1.00, 0.59},
			},]]--
			Power = {
				MANA = {0.31, 0.45, 0.63},
				RAGE = {0.69, 0.31, 0.31},
				FOCUS = {0.71, 0.43, 0.27},
				ENERGY = {0.65, 0.63, 0.35},
				RUNES = {0.55, 0.57, 0.61},
				RUNIC_POWER = {0, 0.82, 1},
				AMMOSLOT = {0.8, 0.6, 0},
				FUEL = {0, 0.55, 0.5},
			},
			Smooth = {
				[1] = {0.69, 0.31, 0.31}, -- Low Health
				[2] = {0.69, 0.69, 0.31}, -- Mid Health
				[3] = {0.31, 0.69, 0.31}, -- High Health
			},
			CombatText = {
				DAMAGE = {0.69, 0.31, 0.31},
				CRUSHING = {0.69, 0.31, 0.31},
				CRITICAL = {0.69, 0.31, 0.31},
				GLANCING = {0.69, 0.31, 0.31},
				STANDARD = {0.84, 0.75, 0.65},
				IMMUNE = {0.84, 0.75, 0.65},
				ABSORB = {0.84, 0.75, 0.65},
				BLOCK = {0.84, 0.75, 0.65},
				RESIST = {0.84, 0.75, 0.65},
				MISS = {0.84, 0.75, 0.65},
				HEAL = {0.33, 0.59, 0.33},
				CRITHEAL = {0.33, 0.59, 0.33},
				ENERGIZE = {0.31, 0.45, 0.63},
				CRITENERGIZE = {0.31, 0.45, 0.63},
			},
			--merker
			HolyPowerBar = {
				[1] = {0.90, 0.88, 0.06},
				[2] = {0.90, 0.88, 0.06},
				[3] = {0.90, 0.88, 0.06},
				[4] = {0.90, 0.88, 0.06},
				[5] = {0.90, 0.88, 0.06},
			},
			ArcaneChargesBar = {
				[1] = {0.12, 0.58, 0.89},
				[2] = {0.12, 0.58, 0.89},
				[3] = {0.12, 0.58, 0.89},
				[4] = {0.12, 0.58, 0.89},
				[5] = {0.12, 0.58, 0.89},
				[6] = {0.12, 0.58, 0.89},
			},
			ChiBar = {
				[1] = {0.00, 1.00, 0.59},
				[2] = {0.00, 1.00, 0.59},
				[3] = {0.00, 1.00, 0.59},
				[4] = {0.00, 1.00, 0.59},
				[5] = {0.00, 1.00, 0.59},
				[6] = {0.00, 1.00, 0.59},
			},
			WarlockBar = {
				["Fury"] = {0.8, 0.1, 0.1},
				["Shard1"] = {0.57, 0.22, 1},
				["Shard2"] = {0.57, 0.22, 1},
				["Shard3"] = {0.57, 0.22, 1},
				["Shard4"] = {0.57, 0.22, 1},
				["Ember1"] = {0.69, 0.31, 0.31},
				["Ember2"] = {0.69, 0.31, 0.31},
				["Ember3"] = {0.69, 0.31, 0.31},
				["Ember4"] = {0.69, 0.31, 0.31},
			},
			Runes = {
					[1] = {0.69, 0.31, 0.31}, -- Blood Rune
					[3] = {0.33, 0.59, 0.33}, -- Unholy Rune
					[2] = {0.31, 0.45, 0.63}, -- Frost Rune
					[4] = {0.84, 0.75, 0.65}, -- Death Rune
				},
			ComboPoints = {
				[1] = {0.95, 0.86, 0.16},
				[2] = {0.95, 0.86, 0.16},
				[3] = {0.95, 0.86, 0.16},
				[4] = {0.95, 0.86, 0.16},
				[5] = {0.95, 0.86, 0.16},
			},
			Totems = {
				[1] = {0.04, 0.39, 0.98}, -- Fire
				[2] = {0.04, 0.39, 0.98}, -- Earth
				[3] = {0.04, 0.39, 0.98}, -- Water
				[4] = {0.04, 0.39, 0.98}, -- Air
			},
			LevelDiff = {
				[1] = {0.69, 0.31, 0.31}, -- Target Level >= 5
				[2] = {0.71, 0.43, 0.27}, -- Target Level >= 3
				[3] = {0.84, 0.75, 0.65}, -- Target Level <> 2
				[4] = {0.33, 0.59, 0.33}, -- Target Level GreenQuestRange
				[5] = {0.55, 0.57, 0.61}, -- Low Level Target
			},
			Misc = {
				Tapped = {0.7, 0.7, 0.7},
				Hostile = {0.7, 0.3, 0.3},
				Neutral = {0.7, 0.7, 0.3},
				Friendly = {0.3, 0.7, 0.3},
			},
		},
	}
}

-- ####################################################################################################################
-- ##### Framework Events #############################################################################################
-- ####################################################################################################################

function module:LoadOptions()
	local options = {
		header = self:NewHeader("Unit Frames", 1),
		Settings = self:CreateSettings(2),
		Colors = self:CreateColorOptions(3),
		Layout = self:CreateImportExportOptions(4),
		-- XP_Rep = self:CreateXpRepOptions(5),
	}

	for index, unit in pairs(module.units) do
		options[unit] = self:CreateUnitOptions(unit, index)
	end

	return options
end

function module:Refresh()
	for _, unit in pairs(module.unitsSpawn) do
		self.ToggleUnit(unit)
		self.ApplySettings(unit)
	end
end

function module:OnInitialize()
	--LUI:NewNamespace(self, true)
	LUI:RegisterModule(module)
end

function module:OnEnable()
	LUI.Profiler.TraceScope(module, "Unitframes", "LUI", 2)
	for _, unit in pairs(module.unitsSpawn) do module.ToggleUnit(unit) end
end

function module:OnDisable()
	for _, unit in pairs(module.unitsSpawn) do module.ToggleUnit(unit, false) end
end
